@@ -290,17 +290,26 @@ class Primitive(Atoms):
             raise ValueError
 
     def _supercell_to_primitive_map(self, pos):
-        inv_F = np.linalg.inv(self._primitive_matrix)
+        frac_pos = np.dot(pos, np.linalg.inv(self._primitive_matrix).T)
+
+        p2s_positions = frac_pos[self._p2s_map]
         s2p_map = []
-        for i in range(pos.shape[0]):
-            s_pos = np.dot(pos[i], inv_F.T)
-            for j in self._p2s_map:
-                p_pos = np.dot(pos[j], inv_F.T)
-                diff = p_pos - s_pos
-                diff -= np.rint(diff)
-                if (abs(diff) < self._symprec).all():
-                    s2p_map.append(j)
-                    break
+        for s_pos in frac_pos:
+
+            # Compute distances from s_pos to all positions in _p2s_map.
+            frac_diffs = p2s_positions - s_pos
+            frac_diffs -= np.rint(frac_diffs)
+
+            cart_diffs = np.dot(frac_diffs, self.cell)
+            distances = np.sqrt((cart_diffs**2).sum(axis=1))
+
+            # Find the one distance that's short enough.
+            matches = (distances < self._symprec).tolist()
+            assert matches.count(True) == 1
+            index = matches.index(True)
+
+            s2p_map.append(self._p2s_map[index])
+
         self._s2p_map = np.array(s2p_map, dtype='intc')
 
     def _primitive_to_primitive_map(self):
@@ -532,15 +541,17 @@ def _get_smallest_vectors(supercell, primitive, symprec):
     # shape: (size_super, size_prim, 27, 3)
     candidate_carts = np.dot(candidate_fracs, reduced_bases)
     # shape: (size_super, size_prim, 27)
-    lengths = np.sqrt(np.sum(candidate_carts**2, axis=-1))
+    lengths = np.array(np.sqrt(np.sum(candidate_carts**2, axis=-1)),
+                       dtype='double', order='C')
 
     # Create the output, initially consisting of all candidate vectors scaled
     # by the primitive cell.
     #
     # shape: (size_super, size_prim, 27, 3)
-    candidate_vectors = np.dot(
+    candidate_vectors = np.array(np.dot(
         candidate_fracs,
-        reduced_bases.dot(np.linalg.inv(primitive.get_cell())))
+        reduced_bases.dot(np.linalg.inv(primitive.get_cell()))),
+                                 dtype='double', order='C')
 
     # The last final bits are done in C.
     #
